/***************************************************************************
 *   Copyright (C) 2005 by Andreas Silberstorff   *
 *   ml@andreas-silberstorff.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/

#include "kalva.h"
#include "kalvasidebar.h"
#include "settings.h"
#include "prefs.h"
#include "kalva_channellist.h"
#include "serialbrowser.h"

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qtimer.h>
#include <qiconset.h>
#include <qptrlist.h>

#include <kiconloader.h>
#include <klocale.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <kconfigdialog.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kdebug.h>
#include <kstandarddirs.h>
#include <ktip.h>
#include <kmultitabbar.h>

#include <kparts/componentfactory.h>
#include "kalvaview.h"
#include "systray.h"
#include <kchlstfilterplugin.h>

Kalva::Kalva()
    : KMainWindow(0, "Kalva", WDestructiveClose),
      m_view(new KalvaView(this)),
      m_printer(0)
{
   //readSettings();

    setAcceptDrops(true);
    setCentralWidget(m_view);
    setMinimumSize( QSize(  (m_view->width() + 10), (m_view->height() + 20) ) );
    resize( minimumSizeHint () );

    setupActions();
    setupPlugins();
    statusBar()->show();

    readConfig();
    setupSystemTray();

    if (m_startDocked)
       hide();
    else
       tipOfTheDay();
    m_shuttingDown  = false;
}

Kalva::~Kalva()
{
}

void Kalva::setupActions()
{
    (void)new KAction(
           i18n("&Save"), 0,
           m_view->channellist(),    SLOT(slotSaveChannellist()),
           actionCollection(),       "save_channellist");
    KStdAction::quit(
           this,                     SLOT(slotQuit()),
           actionCollection());
    m_toolbarAction = KStdAction::showToolbar(
           this,                     SLOT(optionsShowToolbar()),
           actionCollection());
    m_statusbarAction = KStdAction::showStatusbar(
           this,                     SLOT(optionsShowStatusbar()),
           actionCollection());
    KStdAction::keyBindings(
           this,                     SLOT(optionsConfigureKeys()),
           actionCollection());
    KStdAction::configureToolbars(
           this,                     SLOT(optionsConfigureToolbars()),
           actionCollection());
    KStdAction::preferences(
           this,                     SLOT(optionsPreferences()),
           actionCollection());
    m_toggleDockOnCloseAction =	new KToggleAction(
           i18n("&Stay in System Tray on Close"),
	   0,
           actionCollection(),
           "dockOnClose");
    m_toggleSystemTrayAction = new KToggleAction(
           i18n("&Dock in System Tray"),
           0,
           actionCollection(),
           "toggleSystemTray");
    KStdAction::tipOfDay(
           this,                     SLOT( slotTipOfTheDay() ),
           actionCollection()
           )->setWhatsThis(
              i18n("This shows useful tips on the use of this application.")
           );

    connect (
           m_toggleSystemTrayAction, SIGNAL(toggled(bool)),
   	   this,                     SLOT(slotToggleSystemTray(bool)));
    // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));

    QTimer::singleShot( 200, this, SLOT(tipOfTheDay()) );
}

void Kalva::setupPlugins()
{
    // Sets up the plugin interface, and load the plugins
    m_pluginInterface = new QObject( this, "m_pluginInterface" );
    new MyChannellistInterface( m_view,
                                m_pluginInterface,
                                "channellist interface" );

    QString uirc = locate("appdata", "kalvaui.rc");
    kdDebug() << "kalvaui.rc = " << uirc << endl;
    createGUI(uirc, true);
    loadPlugins();
}

void Kalva::tipOfTheDay()
{
  KTipDialog::showTip( this );
}

void Kalva::slotTipOfTheDay()
{
  KTipDialog::showTip( this, QString::null,  true );
}


void Kalva::optionsShowToolbar()
{
    if (m_toolbarAction->isChecked())  {
        toolBar()->show();
    }  else  {
        toolBar()->hide();
    }
}

void Kalva::optionsShowStatusbar()
{
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}

void Kalva::optionsConfigureKeys()
{
   KKeyDialog::configure(actionCollection());
}

void Kalva::optionsConfigureToolbars()
{
   saveMainWindowSettings( KGlobal::config(), "MainWindow" );
   KEditToolbar dlg(actionCollection());
   connect(&dlg,SIGNAL(newToolbarConfig()),this,SLOT(newToolbarConfig()));
   dlg.exec();
}

void Kalva::newToolbarConfig()
{
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
   //createGui();
   KMessageBox::detailedSorry   (
       this,
       i18n("The changes will not show up untill you restart Kalva."), 
       i18n("<qt>If you are an experienced kde coder I want to invite you to take a close look at Kalva's sources. Please through me a line if you know how to provide immediate rebuilding the toolbar without loosing the actions load via plugins...</qt>"), 
       i18n("Changes will only take effect on restart.")
       );
    applyMainWindowSettings(KGlobal::config(), autoSaveGroup());
}

void Kalva::optionsPreferences()
{
	// The preference dialog is derived from prefs-base.ui
        // which is subclassed into Prefs
	//
	// compare the names of the widgets in the .ui file 
	// to the names of the variables in the .kcfg file
        KConfigDialog *dialog = new KConfigDialog(
              this, "settings", Settings::self(), KDialogBase::Swallow);
        dialog->addPage(new Prefs(), i18n("General"), "package_settings");
        connect(dialog, SIGNAL(settingsChanged()),
                m_view, SLOT(settingsChanged()));
        dialog->show();
}

void Kalva::changeStatusbar(const QString& text)
{
    // display the text on the statusbar
    statusBar()->message(text, 2000);
}

void Kalva::changeCaption(const QString& text)
{
    // display the text on the caption
    setCaption(text);
}

void Kalva::loadPlugins()
{
    KalvaChannellist* channellist = m_view->channellist();
    KTrader::OfferList offers =
          KTrader::self()->query("KChlstFilterPlugin/Plugin");

    KTrader::OfferList::ConstIterator iter;
    for(iter = offers.begin(); iter != offers.end(); ++iter) {
        KService::Ptr service = *iter;
        int errCode = 0;
        KChlstFilterPlugin::Plugin* plugin =
            KParts::ComponentFactory::createInstanceFromService<KChlstFilterPlugin::Plugin>
            ( service, m_pluginInterface, 0, QStringList(), &errCode);
        // here we ought to check the error code.

        if (plugin) {
            guiFactory()->addClient(plugin);

            connect(plugin,      SIGNAL(chlstImported(const QStringList& )),
                    channellist, SLOT(slotImport(const QStringList& )));

            kdDebug() << "KChlstFilterPlugin: Loaded plugin "
                      << plugin->name() << endl;
        }

    }
}

void Kalva::readConfig()
{
    KConfigGroup kalvaConfig(KGlobal::config(), "Kalva");
    m_startDocked = kalvaConfig.readBoolEntry("StartDocked", false);
    m_toggleSystemTrayAction->setChecked(
           kalvaConfig.readBoolEntry("DockInSystemTray", true));
    m_toggleDockOnCloseAction->setChecked(
           kalvaConfig.readBoolEntry("DockOnClose", true));

    m_view->sidebar()->readConfig( kalvaConfig );
    QString lastStation = kalvaConfig.readEntry("lastStation");

    kdDebug() << "read last station as "
                      << lastStation << endl;

    m_view->channellist()->setStation( lastStation );
    restoreWindowSize(KGlobal::config());
}

void Kalva::saveConfig()
{
    KConfigGroup kalvaConfig(KGlobal::config(), "Kalva");
    kalvaConfig.writeEntry( "StartDocked",
                            m_startDocked);
    kalvaConfig.writeEntry( "DockInSystemTray",
                            m_toggleSystemTrayAction->isChecked());
    kalvaConfig.writeEntry( "DockOnClose",
                            m_toggleDockOnCloseAction->isChecked());
    saveWindowSize(KGlobal::config());

    m_view->sidebar()->setConfig( kalvaConfig );

    KGlobal::config()->sync();
}

bool Kalva::queryExit()
{
    m_startDocked = !isVisible();
    hide();

    delete m_systemTray;
    m_systemTray = 0;

    saveConfig();
    return true;
}

bool Kalva::queryClose()
{
    if(!m_shuttingDown &&
       !kapp->sessionSaving() &&
       m_systemTray &&
       m_toggleDockOnCloseAction->isChecked())
    {
	KMessageBox::information(this,
	    i18n("<qt>Closing the main window will keep Kalva running in the system tray. "
		 "Use Quit from the File menu to quit the application.</qt>"),
	    i18n("Docking in System Tray"), "hideOnCloseInfo");
	hide();
	return false;
    }
    else
	return true;
}


void Kalva::setupSystemTray()
{
   if(m_toggleSystemTrayAction && m_toggleSystemTrayAction->isChecked()) {
      m_systemTray = new SystemTray(this, "systemTray");
      m_systemTray->show();
      m_toggleDockOnCloseAction->setEnabled(true);
      connect(m_systemTray, SIGNAL(quitSelected()), this, SLOT(slotAboutToQuit()));
   }
   else {
      m_systemTray = 0;
      m_toggleDockOnCloseAction->setEnabled(false);
   }
}

void Kalva::slotToggleSystemTray(bool enabled)
{
    if(enabled && !m_systemTray)
	setupSystemTray();
   else if(!enabled && m_systemTray) {
      delete m_systemTray;
      m_systemTray = 0;
      m_toggleDockOnCloseAction->setEnabled(false);
   }
}

void Kalva::slotAboutToQuit()
{
    m_shuttingDown = true;
}

void Kalva::slotQuit()
{
    m_shuttingDown = true;
    kapp->quit();
}

#include "kalva.moc"

